/**************************************************************************

Copyright (c) 2002 Dmitry Vukolov <dvukol@rosmail.ru>
Portions copyright (c) 2000 Palm, Inc. or its subsidiaries.
All rights reserved.

This is free software, licensed under the GNU Public License v2.
See the file COPYING for details.

**************************************************************************/

#include <PalmOS.h>
#include "addressdb.h"
#include "commondb.h"
#include "convert.h"

/**********************************************************
        Private definitions:
***********************************************************/
void AddrConvAppInfo(MemPtr AppInfoP);
void AddrConvRecord(MemPtr RecordP);

#define addrDBType      'DATA'
#define addrDBCreator   sysFileCAddress

#define BitAtPosition(pos)              ((UInt32)1 << (pos))
#define GetBitMacro(bitfield, index)    ((bitfield) & BitAtPosition(index))

typedef union {
	UInt32 allBits;
} AddrDBRecordFlags;

typedef enum {
	name,
	firstName,
	company,
	phone1,
	phone2,
	phone3,
	phone4,
	phone5,
	address,
	city,
	state,
	zipCode,
	country,
	title,
	custom1,
	custom2,
	custom3,
	custom4,
	note,			// This field is assumed to be < 4K
	addressFieldsCount
} AddressFields;

typedef UInt32 AddrOptionsType;

// The following structure doesn't really exist.  The first field
// varies depending on the data present.  However, it is convient
// (and less error prone) to use when accessing the other information.
typedef struct {
	AddrOptionsType options;
	AddrDBRecordFlags flags;
	UInt8 companyFieldOffset;
	Char firstField;
} AddrPackedDBRecordType;

typedef AddrPackedDBRecordType *AddrPackedDBRecordPtr;

typedef UInt32 AddrDBFieldLabelsDirtyFlags;
typedef UInt8 AddrDBMisc;

#define addrLabelLength			16	// 15 Chars + 1 null terminator

#define firstAddressField       	name
#define firstPhoneField		        phone1
#define lastPhoneField		        phone5
#define numPhoneLabels		        8
#define numPhoneFields		        (lastPhoneField - firstPhoneField + 1)
#define numPhoneLabelsStoredFirst	numPhoneFields
#define numPhoneLabelsStoredSecond	(numPhoneLabels - numPhoneLabelsStoredFirst)

#define numAddrLabels                   (addressFieldsCount + numPhoneLabelsStoredSecond)

typedef Char addressLabel[addrLabelLength];

typedef struct {
	UInt16 renamedCategories;	// bitfield of categories with a different name
	Char categoryLabels[dmRecNumCategories][dmCategoryLength];
	UInt8 categoryUniqIDs[dmRecNumCategories];
	UInt8 lastUniqID;
	UInt8 reserved1;
	UInt16 reserved2;
	AddrDBFieldLabelsDirtyFlags dirtyFieldLabels;
	addressLabel fieldLabels[numAddrLabels];
	CountryType country;	// Country the database (labels) is formatted for
	UInt8 reserved;
	AddrDBMisc misc;
} AddrAppInfoType;

typedef AddrAppInfoType *AddrAppInfoPtr;


/**********************************************************
        Converting Functions:
***********************************************************/

void AddrConvAppInfo(MemPtr RecordP)
{
	Int16 index;
	Char *p;
	AddrAppInfoPtr appInfoP;

	appInfoP = (AddrAppInfoPtr) RecordP;

	p = (Char *) appInfoP->categoryLabels;
	for (index = 0; index < dmRecNumCategories; index++) {
		ConvertDBStr(p, appInfoP);
		p += dmCategoryLength;
	}

	p = (Char *) appInfoP->fieldLabels;
	for (index = 0; index < numAddrLabels; index++) {
		ConvertDBStr(p, appInfoP);
		p += addrLabelLength;
	}
}


void AddrConvRecord(MemPtr RecordP)
{
	Int16 index;
	UInt32 flags;
	Char *p;
	AddrPackedDBRecordPtr record;

	record = (AddrPackedDBRecordPtr) RecordP;
	flags = record->flags.allBits;
	p = &record->firstField;

	for (index = firstAddressField; index < addressFieldsCount;
	     index++) {
		if (GetBitMacro(flags, index) != 0) {
			ConvertDBStr(p, record);
			p += StrLen(p) + 1;
		}
	}
}


void AddrConvert()
{
	ConvertDB(addrDBType, addrDBCreator, AddrConvAppInfo,
		  AddrConvRecord);
}
