/**************************************************************************

Copyright (c) 2002 Dmitry Vukolov <dvukol@rosmail.ru>
Portions copyright (c) 2000 Palm, Inc. or its subsidiaries.
All rights reserved.

This is free software, licensed under the GNU Public License v2.
See the file COPYING for details.

**************************************************************************/

#include <PalmOS.h>
#include "datedb.h"
#include "commondb.h"
#include "convert.h"

/**********************************************************
        Private definitions:
***********************************************************/
//void DateConvAppInfo(MemPtr AppInfoP);
void DateConvRecord(MemPtr RecordP);

#define dateDBType      'DATA'
#define dateDBCreator   sysFileCDatebook

// This structure describes what information is present in an
// AppointmentPackedDBRecordType
typedef struct {
	unsigned when:1;	// set if when info changed (ApptChangeRecord)
	unsigned alarm:1;	// set if record contains alarm info
	unsigned repeat:1;	// set if record contains repeat info
	unsigned note:1;	// set if record contains a note
	unsigned exceptions:1;	// set if record contains exceptions list
	unsigned description:1;
} ApptDBRecordFlags;

typedef struct {
	TimeType startTime;	// Time the appointment starts
	TimeType endTime;	// Time the appointment ends
	DateType date;		// date of appointment
} ApptDateTimeType;

// The following structure doesn't really exist.  The first field
// varies depending on the data present.  However, it is convient
// (and less error prone) to use when accessing the other information.
typedef struct {
	ApptDateTimeType when;
	ApptDBRecordFlags flags;	// A flag set for each  datum present
	Char firstField;
	UInt8 reserved;
} ApptPackedDBRecordType;

typedef ApptPackedDBRecordType *ApptPackedDBRecordPtr;

typedef struct {
	UInt16 renamedCategories;	// bitfield of categories with a different name
	Char categoryLabels[dmRecNumCategories][dmCategoryLength];
	UInt8 categoryUniqIDs[dmRecNumCategories];
	UInt8 lastUniqID;
	UInt8 reserved1;
	UInt16 reserved2;
	UInt8 startOfWeek;
	UInt8 reserved;
} ApptAppInfoType;

typedef ApptAppInfoType *ApptAppInfoPtr;

typedef enum alarmTypes { aauMinutes, aauHours, aauDays } AlarmUnitType;

typedef struct {
	Int8 advance;		// Alarm advance (-1 = no alarm)
	AlarmUnitType advanceUnit;	// minutes, hours, days
} AlarmInfoType;

enum repeatTypes {
	repeatNone,
	repeatDaily,
	repeatWeekly,
	repeatMonthlyByDay,
	repeatMonthlyByDate,
	repeatYearly
};
typedef enum repeatTypes RepeatType;

typedef struct {
	RepeatType repeatType;	// daily, weekly, monthlyByDay, etc.
	UInt8 reserved1;
	DateType repeatEndDate;	// minus one if forever
	UInt8 repeatFrequency;	// i.e. every 2 days if repeatType daily
	UInt8 repeatOn;		// monthlyByDay and repeatWeekly only
	UInt8 repeatStartOfWeek;	// repeatWeekly only
	UInt8 reserved2;
} RepeatInfoType;

// This is the structure for a repeating appointment's exceptions list.  The
// exceptions list is a variable length list of dates that the repeating appointment
// should not appear on.  The list is in chronological order.
//
typedef struct {
	UInt16 numExceptions;
	DateType exception;
} ExceptionsListType;


/**********************************************************
        Converting Functions:
***********************************************************/

/*
Next function is not needed for DateBook v3.0, because though category
labels are declared in its source files they are not actually used
*/
/*void DateConvAppInfo(MemPtr RecordP)
{
        Int16 index;
        Char *p;
        ApptAppInfoPtr appInfo;

        appInfo = (ApptAppInfoPtr) RecordP;

        p = (Char*)appInfoP->categoryLabels;
        for(index = 0; index < dmRecNumCategories; index++) {
                ConvertDBStr(p, appInfoP);
                p += dmCategoryLength;
        }
}*/

void DateConvRecord(MemPtr RecordP)
{
	ApptDBRecordFlags flags;
	Char *p;
	ApptPackedDBRecordPtr record;

	record = (ApptPackedDBRecordPtr) RecordP;
	flags = record->flags;
	p = &record->firstField;
	if (flags.alarm)
		p += sizeof(AlarmInfoType);
	if (flags.repeat)
		p += sizeof(RepeatInfoType);
	if (flags.exceptions)
		p += sizeof(UInt16) +
		    (((ExceptionsListType *) p)->numExceptions *
		     sizeof(DateType));
	if (flags.description) {
		ConvertDBStr(p, record);
		p += StrLen(p) + 1;
	}
	if (flags.note)
		ConvertDBStr(p, record);
}

void DateConvert()
{
	ConvertDB(dateDBType, dateDBCreator, NULL, DateConvRecord);
}
